@@ -1,13 +1,19 @@
-from typing import *
-from ideaseed.constants import C_PRIMARY, RELEASES_RSS_URL
-from xml.dom.minidom import parseString as parse_xml
-import cli_box
-import requests
-from ideaseed.utils import ask, dye
-import inquirer as q
-from semantic_version import Version
-import subprocess
 import re
+import subprocess
+from xml.dom.minidom import parseString as parse_xml
+
+import requests
+from rich import print
+from rich.markdown import Markdown
+from rich.rule import Rule
+from semantic_version import Version
+
+from ideaseed.constants import RELEASES_RSS_URL, VERSION
+from ideaseed.ui import \
+    FramelessCodeBlock  # markdown with no ugly frame around code blocks
+from ideaseed.utils import answered_yes_to, ask
+
+Markdown.elements["code_block"] = FramelessCodeBlock
 
 
 def get_latest_version() -> Version:
@@ -24,10 +30,16 @@ def get_latest_version() -> Version:
     return Version(version)
 
 
+def get_release_notes() -> str:
+    return requests.get(
+        "https://raw.githubusercontent.com/ewen-lbh/ideaseed/master/CHANGELOG.md"
+    ).text
+
+
 def get_changelog_heading_anchor(release_notes: str, upgrade_to: Version) -> str:
     """
     Get the changelog heading anchor for github.
-    >>> get_changelog_heading_anchor(Version('0.8.0'))
+    >>> get_changelog_heading_anchor(get_release_notes(), Version('0.8.0'))
     '080---2020-06-20'
     """
     pattern = re.compile(r"## \[" + str(upgrade_to) + r"\] - (.+)")
@@ -35,12 +47,6 @@ def get_changelog_heading_anchor(release_notes: str, upgrade_to: Version) -> str
     return f"{str(upgrade_to).replace('.', '')}---{date}"
 
 
-def get_release_notes() -> str:
-    return requests.get(
-        "https://raw.githubusercontent.com/ewen-lbh/ideaseed/master/CHANGELOG.md"
-    ).text
-
-
 def get_release_notes_for_version(release_notes: str, version: Version) -> str:
     in_target_version = False
     ret = ""
@@ -57,13 +63,13 @@ def get_release_notes_for_version(release_notes: str, version: Version) -> str:
     return ret
 
 
-def get_versions_list_from_release_notes(release_notes: str) -> List[Version]:
+def get_versions_list_from_release_notes(release_notes: str) -> list[Version]:
     # Declarations
     pattern = re.compile(
         r"\[Unreleased\]\: https\:\/\/github\.com\/ewen\-lbh\/ideaseed\/compare\/v\d+\.\d+\.\d+\.\.\.HEAD"
     )
     extract_version_pattern = re.compile(r"\[(\d+\.\d+\.\d+)\]: https://")
-    version_strings: List[str] = []
+    version_strings: list[str] = []
     # Tracking varialbe
     in_links_section = False
     for line in release_notes.splitlines():
@@ -90,6 +96,8 @@ def get_release_notes_between_versions(
         for v in get_versions_list_from_release_notes(release_notes)
         if version_from < v <= version_to
     ]
+    # Order by most recent first
+    versions = reversed(versions)
     catd_release_notes = ""
     for version in versions:
         catd_release_notes += f"## {version}"
@@ -102,53 +110,28 @@ def get_release_notes_link(release_notes: str, upgrade_to: Version) -> str:
     return f"https://github.com/ewen-lbh/ideaseed/tree/master/CHANGELOG.md#{anchor}"
 
 
-def notification(upgrade_from: Version, upgrade_to: Version) -> str:
-    return cli_box.rounded(
-        f"""==== Update available! ====
+def notification(upgrade_from: Version, upgrade_to: Version) -> None:
+    print(Rule("Update available!"))
+    print(
+        f"""A new version of ideaseed is available for download:
+[blue bold]{upgrade_from}[/] [magenta]->[/] [blue bold]{upgrade_to}[/]
 
-A new version of ideaseed is available for download:
-{upgrade_from} -> {upgrade_to}
-"""
+Use [blue bold]ideaseed update[/] to see what changed and do the update."""
     )
-
-
-def render_markdown(text: str) -> str:
-    heading = re.compile(r"(#+)\s*(.+)")
-    list_item = re.compile(r"(\s*)-\s*(.+)")
-    image = re.compile(r"!\[(.+)\]\((.+)\)")
-    code = re.compile(r"`([^`]+)`")
-    # link = re.compile(r'\[(.+)\]\((.+)\)')
-    rendered = ""
-    for line in text.splitlines():
-        if heading.match(line):
-            match = heading.search(line)
-            rendered_line = dye(match.group(2), style="bold")
-        elif list_item.match(line):
-            match = list_item.search(line)
-            rendered_line = match.group(1) + dye("• ", style="dim") + match.group(2)
-        else:
-            rendered_line = line
-        rendered_line = image.sub(dye(r"(image: \1)", style="dim"), rendered_line)
-        rendered_line = code.sub(dye(r" \1 ", bg=0xDEDEDE), rendered_line)
-        # rendered_line = link.sub(dye(r' (link: \1)', style="dim"), rendered_line)
-        rendered += rendered_line + "\n"
-    return rendered
+    print(Rule())
 
 
 def prompt(upgrade_from: Version, upgrade_to: Version) -> bool:
     """
     Returns ``True`` if the user wants to upgrade, ``False`` otherwise.
     """
-    answer = ask(
-        q.List(
-            "ans",
-            message=f"Upgrade to v{upgrade_to} now?",
-            choices=["Yes", "What has changed?", "No"],
-        )
-    )
-    if answer == "What has changed?":
+    if not answered_yes_to(
+        f"Upgrade to v{upgrade_to} now? (you will be able to view the release notes)",
+    ):
+        return False
+
+    if answered_yes_to("List what changed?"):
         release_notes = get_release_notes()
-        url = get_release_notes_link(release_notes, upgrade_to)
         all_versions = get_versions_list_from_release_notes(release_notes)
         # If the version jump is more than one version, print concatednated release notes
         # so that the user can get all of the changes.
@@ -164,23 +147,27 @@ def prompt(upgrade_from: Version, upgrade_to: Version) -> bool:
         else:
             notes = get_release_notes_for_version(release_notes, upgrade_to)
         print(
-            f"""\
-Release notes for v{upgrade_from} -> v{upgrade_to}
-====================================
-
-{render_markdown(notes).strip()}
-
----------------------------------------------
-To see images, you can also read this online:
-{url}
-"""
+            Rule(
+                f"Release notes for [bold blue]{upgrade_from}[/] [magenta]->[/] [bold blue]{upgrade_to}[/]"
+            )
         )
-        return ask(q.Confirm("ans", message="Upgrade now?"))
-    else:
-        return answer == "Yes"
+        print(Markdown(notes))
+        return answered_yes_to("Update now?")
+
+    return True
 
 
-def upgrade(upgrade_to: Version):
+def upgrade(upgrade_from: Version, upgrade_to: Version):
     cmd = ["pip", "install", "--upgrade", f"ideaseed=={upgrade_to}"]
     print(f"Running {' '.join(cmd)}...")
     subprocess.run(cmd)
+
+
+def check_and_prompt():
+    latest_version = get_latest_version()
+    if latest_version > VERSION and prompt(VERSION, latest_version):
+        upgrade(VERSION, latest_version)
+        return
+    else:
+        # FIXME: also gets printed out if answered no to the prompt
+        print("Great, you're already up to date!")
